@@ -7,6 +7,7 @@ import logging
 import os
 import re
 import sys
+from contextlib import suppress
 from typing import (
     Any,
     Callable,
@@ -21,9 +22,8 @@ from typing import (
     Union,
 )
 
-from ._common import lenient_check, parser_context
+from ._common import is_dataclass_like, is_subclass, lenient_check, parser_context
 from .actions import (
-    Action,
     ActionConfigFile,
     ActionParser,
     _ActionConfigLoad,
@@ -39,6 +39,7 @@ from .actions import (
     parent_parsers,
     previous_config,
 )
+from .deprecated import ParserDeprecations
 from .formatters import DefaultHelpFormatter, empty_help, get_env_var
 from .jsonnet import ActionJsonnet, ActionJsonnetExtVars
 from .jsonschema import ActionJsonSchema
@@ -70,18 +71,15 @@ from .optionals import (
     import_jsonnet,
 )
 from .parameter_resolvers import UnknownDefault
-from .signatures import SignatureArguments, is_pure_dataclass
+from .signatures import SignatureArguments
 from .typehints import ActionTypeHint, is_subclass_spec
-from .typing import is_final_class
 from .util import (
-    ParserError,
     Path,
+    argument_error,
     change_to_path_dir,
     get_private_kwargs,
     identity,
-    is_subclass,
     return_parser_if_captured,
-    usage_and_exit_error_handler,
 )
 
 __all__ = ['ActionsContainer', 'ArgumentParser']
@@ -93,7 +91,7 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
     _action_groups: Sequence['_ArgumentGroup']  # type: ignore
 
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
         self.register('type', None, identity)
         self.register('action', 'parsers', _ActionSubCommands)
@@ -118,14 +116,10 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
             if is_subclass(kwargs['action'], ActionConfigFile) and any(isinstance(a, ActionConfigFile) for a in self._actions):
                 raise ValueError('A parser is only allowed to have a single ActionConfigFile argument.')
         if 'type' in kwargs:
-            if is_final_class(kwargs['type']) or is_pure_dataclass(kwargs['type']):
+            if is_dataclass_like(kwargs['type']):
                 theclass = kwargs.pop('type')
                 nested_key = re.sub('^--', '', args[0])
-                if is_final_class(theclass):
-                    kwargs.pop('help', None)
-                    self.add_class_arguments(theclass, nested_key, **kwargs)
-                else:
-                    self.add_dataclass_arguments(theclass, nested_key, **kwargs)
+                self.add_dataclass_arguments(theclass, nested_key, **kwargs)
                 return _find_action(parser, nested_key)
             if ActionTypeHint.is_supported_typehint(kwargs['type']):
                 args = ActionTypeHint.prepare_add_argument(
@@ -150,7 +144,7 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
         return action
 
 
-    def add_argument_group(self, *args, name: Optional[str] = None, **kwargs):
+    def add_argument_group(self, *args, name: Optional[str] = None, **kwargs) -> '_ArgumentGroup':
         """Adds a group to the parser.
 
         All the arguments from `argparse.ArgumentParser.add_argument_group
@@ -183,8 +177,8 @@ class _ArgumentGroup(ActionsContainer, argparse._ArgumentGroup):
     parser: Optional['ArgumentParser'] = None
 
 
-class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser):
-    """Parser for command line, yaml/jsonnet files and environment variables."""
+class ArgumentParser(ParserDeprecations, ActionsContainer, ArgumentLinking, argparse.ArgumentParser):
+    """Parser for command line, configuration files and environment variables."""
 
     formatter_class: Type[DefaultHelpFormatter]
     groups: Optional[Dict[str, '_ArgumentGroup']] = None
@@ -195,8 +189,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         self,
         *args,
         env_prefix: Union[bool, str] = True,
-        error_handler: Optional[Callable[['ArgumentParser', str], None]] = usage_and_exit_error_handler,
         formatter_class: Type[DefaultHelpFormatter] = DefaultHelpFormatter,
+        exit_on_error: bool = True,
         logger: Union[bool, str, dict, logging.Logger] = False,
         version: Optional[str] = None,
         print_config: Optional[str] = '--print_config',
@@ -206,7 +200,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         default_env: bool = False,
         default_meta: bool = True,
         **kwargs
-    ):
+    ) -> None:
         """Initializer for ArgumentParser instance.
 
         All the arguments from the initializer of `argparse.ArgumentParser
@@ -215,7 +209,6 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
         Args:
             env_prefix: Prefix for environment variables. ``True`` to derive from ``prog``.
-            error_handler: Handler for parsing errors, if None raises :class:`.ParserError` exception.
             formatter_class: Class for printing help messages.
             logger: Configures the logger, see :class:`.LoggerProperty`.
             version: Program version which will be printed by the --version argument.
@@ -229,6 +222,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         super().__init__(*args, formatter_class=formatter_class, logger=logger, **kwargs)
         if self.groups is None:
             self.groups = {}
+        self.exit_on_error = exit_on_error
         self.required_args: Set[str] = set()
         self.save_path_content: Set[str] = set()
         self.default_config_files = default_config_files  # type: ignore
@@ -237,7 +231,6 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         self.env_prefix = env_prefix
         self.parser_mode = parser_mode
         self.dump_header = dump_header
-        self.error_handler = error_handler
         self._print_config = print_config
         if version is not None:
             self.add_argument('--version', action='version', version='%(prog)s '+version, help='Print version and exit.')
@@ -257,7 +250,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         try:
             with patch_namespace(), parser_context(parent_parser=self, lenient_check=True), ActionTypeHint.subclass_arg_context(self):
                 namespace, args = self._parse_known_args(args, namespace)
-        except (argparse.ArgumentError, ParserError) as ex:
+        except argparse.ArgumentError as ex:
             self.error(str(ex), ex)
 
         return namespace, args
@@ -367,7 +360,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             A config object with all parsed values.
 
         Raises:
-            ParserError: If there is a parsing error and error_handler=None.
+            ArgumentError: If the parsing fails error and exit_on_error=True.
         """
         skip_check = get_private_kwargs(kwargs, _skip_check=False)
         return_parser_if_captured(self)
@@ -427,7 +420,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             A config object with all parsed values.
 
         Raises:
-            ParserError: If there is a parsing error and error_handler=None.
+            ArgumentError: If the parsing fails error and exit_on_error=True.
         """
         skip_check, skip_required = get_private_kwargs(kwargs, _skip_check=False, _skip_required=False)
 
@@ -506,7 +499,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             A config object with all parsed values.
 
         Raises:
-            ParserError: If there is a parsing error and error_handler=None.
+            ArgumentError: If the parsing fails error and exit_on_error=True.
         """
         skip_check, skip_subcommands = get_private_kwargs(kwargs, _skip_check=False, _skip_subcommands=False)
 
@@ -538,7 +531,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         with_meta: Optional[bool] = None,
         **kwargs,
     ) -> Namespace:
-        """Parses a configuration file (yaml or jsonnet) given its path.
+        """Parses a configuration file given its path.
 
         Args:
             cfg_path: Path to the configuration file to parse.
@@ -551,7 +544,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             A config object with all parsed values.
 
         Raises:
-            ParserError: If there is a parsing error and error_handler=None.
+            ArgumentError: If the parsing fails error and exit_on_error=True.
         """
         fpath = Path(cfg_path, mode=get_config_read_mode())
         with change_to_path_dir(fpath):
@@ -580,7 +573,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         with_meta: Optional[bool] = None,
         **kwargs,
     ) -> Namespace:
-        """Parses configuration (yaml or jsonnet) given as a string.
+        """Parses configuration given as a string.
 
         Args:
             cfg_str: The configuration content.
@@ -594,7 +587,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             A config object with all parsed values.
 
         Raises:
-            ParserError: If there is a parsing error and error_handler=None.
+            ArgumentError: If the parsing fails error and exit_on_error=True.
         """
         skip_check, fail_no_subcommand = get_private_kwargs(kwargs, _skip_check=False, _fail_no_subcommand=True)
 
@@ -629,7 +622,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         ext_vars: Optional[dict] = None,
         prev_cfg: Optional[Namespace] = None,
     ) -> Namespace:
-        """Loads a configuration string (yaml or jsonnet) into a namespace.
+        """Loads a configuration string into a namespace.
 
         Args:
             cfg_str: The configuration content.
@@ -671,7 +664,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         """
         if 'description' not in kwargs:
             kwargs['description'] = 'For more details of each subcommand, add it as an argument followed by --help.'
-        with parser_context(parent_parser=self):
+        with parser_context(parent_parser=self, lenient_check=True):
             subcommands: _ActionSubCommands = super().add_subparsers(dest=dest, **kwargs)  # type: ignore
         if required:
             self.required_args.add(dest)
@@ -912,10 +905,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             default_config_files += [(None, x) for x in files]
 
         if len(default_config_files) > 0:
-            try:
+            with suppress(TypeError):
                 return [(k, Path(v, mode=get_config_read_mode())) for k, v in default_config_files]
-            except TypeError:
-                pass
         return []
 
 
@@ -983,8 +974,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                             skip_check=skip_check,
                             skip_required=True,
                         )
-                except (TypeError, KeyError, ParserError) as ex:
-                    raise ParserError(f'Problem in default config file "{default_config_file}" :: {ex.args[0]}') from ex
+                except (TypeError, KeyError, argparse.ArgumentError) as ex:
+                    raise argument_error(f'Problem in default config file "{default_config_file}": {ex.args[0]}') from ex
             meta = cfg.get('__default_config__')
             if isinstance(meta, list):
                 meta.append(default_config_file)
@@ -1002,14 +993,19 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
     ## Other methods ##
 
     def error(self, message: str, ex: Optional[Exception] = None) -> NoReturn:
-        """Logs error message if a logger is set, calls the error handler and raises a ParserError."""
+        """Logs error message if a logger is set and exits or raises an ArgumentError."""
         self._logger.error(message)
-        if self._error_handler is not None:
+        if callable(self._error_handler):
             self._error_handler(self, message)
-        if ex is None:
-            raise ParserError(message)
-        else:
-            raise ParserError(message) from ex
+        if not self.exit_on_error:
+            raise argument_error(message) from ex
+        elif 'JSONARGPARSE_DEBUG' in os.environ:
+            self._logger.debug('Debug enabled, thus raising exception instead of exit.')
+            raise argument_error(message) from ex
+        self.print_usage(sys.stderr)
+        args = {'prog': self.prog, 'message': message}
+        sys.stderr.write('%(prog)s: error: %(message)s\n' % args)
+        self.exit(2)
 
 
     def check_config(
@@ -1107,7 +1103,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         components: List[Union[ActionTypeHint, _ActionConfigLoad, _ArgumentGroup]] = []
         for action in filter_default_actions(self._actions):
             if isinstance(action, ActionTypeHint) or \
-               (isinstance(action, _ActionConfigLoad) and is_pure_dataclass(action.basetype)):
+               (isinstance(action, _ActionConfigLoad) and is_dataclass_like(action.basetype)):
                 components.append(action)
 
         if instantiate_groups:
@@ -1195,7 +1191,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                     if isinstance(config_files, list):
                         config_files = [str(x) for x in config_files]
                     note = f'default values below are the ones overridden by the contents of: {config_files}'
-            except ParserError as ex:
+            except argparse.ArgumentError as ex:
                 note = f'tried getting defaults considering default_config_files but failed due to: {ex}'
             group = self._default_config_files_group
             group.description = f'{self._default_config_files}, Note: {note}'
@@ -1204,9 +1200,9 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         return help_str
 
 
-    def print_usage(self, file=None):
+    def print_usage(self, *args, **kwargs) -> None:
         with parser_context(parent_parser=self):
-            return super().print_usage(file)
+            return super().print_usage(*args, **kwargs)
 
 
     def _apply_actions(
@@ -1329,27 +1325,6 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
     ## Properties ##
 
-    @property
-    def error_handler(self) -> Optional[Callable[['ArgumentParser', str], None]]:
-        """Property for the error_handler function that is called when there are parsing errors.
-
-        :getter: Returns the current error_handler function.
-        :setter: Sets a new error_handler function (Callable[self, message:str] or None).
-
-        Raises:
-            ValueError: If an invalid value is given.
-        """
-        return self._error_handler
-
-
-    @error_handler.setter
-    def error_handler(self, error_handler: Optional[Callable[['ArgumentParser', str], None]]):
-        if callable(error_handler) or error_handler is None:
-            self._error_handler = error_handler
-        else:
-            raise ValueError('error_handler can be either a Callable or None.')
-
-
     @property
     def default_config_files(self) -> List[str]:
         """Default config file locations.
