@@ -10,13 +10,13 @@ from dataclasses import dataclass
 from functools import partial
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
+from ._common import is_dataclass_like, is_subclass
 from ._stubs_resolver import get_stub_types
 from .optionals import parse_docs
 from .util import (
     ClassFromFunctionBase,
     LoggerProperty,
     get_import_path,
-    is_subclass,
     iter_to_set_str,
     parse_logger,
     unique,
@@ -80,7 +80,7 @@ def is_staticmethod(attr) -> bool:
 
 
 def is_method(attr) -> bool:
-    return inspect.isfunction(attr) and not is_staticmethod(attr)
+    return (inspect.isfunction(attr) or attr.__class__.__name__ == 'cython_function_or_method') and not is_staticmethod(attr)
 
 
 def is_property(attr) -> bool:
@@ -265,10 +265,7 @@ def get_signature_parameters_and_indexes(component, parent, logger):
         params = params[1:]
     args_idx = get_arg_kind_index(params, kinds.VAR_POSITIONAL)
     kwargs_idx = get_arg_kind_index(params, kinds.VAR_KEYWORD)
-    doc_params = {}
-    for doc in parse_docs(component, parent, logger):
-        for param in doc.params:
-            doc_params[param.arg_name] = param.description
+    doc_params = parse_docs(component, parent, logger)
     for num, param in enumerate(params):
         params[num] = ParamData(
             doc=doc_params.get(param.name),
@@ -328,8 +325,7 @@ def get_kwargs_pop_or_get_parameter(node, component, parent, doc_params, log_deb
 
 
 def is_param_subclass_instance_default(param: ParamData) -> bool:
-    from .signatures import is_pure_dataclass
-    if is_pure_dataclass(type(param.default)):
+    if is_dataclass_like(type(param.default)):
         return False
     from .typehints import ActionTypeHint, get_subclass_types
     class_types = get_subclass_types(param.annotation)
@@ -767,6 +763,37 @@ def get_parameters_by_assumptions(
     return params
 
 
+def get_parameters_from_pydantic(
+    function_or_class: Union[Callable, Type],
+    method_or_property: Optional[str],
+    logger: logging.Logger,
+) -> Optional[ParamList]:
+    from .optionals import pydantic_support
+    if not pydantic_support or method_or_property:
+        return None
+    from pydantic import BaseModel  # pylint: disable=no-name-in-module
+    if not is_subclass(function_or_class, BaseModel):
+        return None
+    params = []
+    doc_params = parse_docs(function_or_class, None, logger)
+    for field in function_or_class.__fields__.values():  # type: ignore
+        if field.required:
+            default = inspect._empty
+        elif field.default_factory:
+            default = field.default_factory()
+        else:
+            default = field.default
+        params.append(ParamData(
+            name=field.name,
+            annotation=field.annotation,
+            default=default,
+            kind=kinds.KEYWORD_ONLY,
+            doc=field.field_info.description or doc_params.get(field.name),
+            component=function_or_class,
+        ))
+    return params
+
+
 def get_signature_parameters(
     function_or_class: Union[Callable, Type],
     method_or_property: Optional[str] = None,
@@ -787,6 +814,9 @@ def get_signature_parameters(
     """
     logger = parse_logger(logger, 'get_signature_parameters')
     try:
+        params = get_parameters_from_pydantic(function_or_class, method_or_property, logger)
+        if params is not None:
+            return params
         visitor = ParametersVisitor(function_or_class, method_or_property, logger=logger)
         return visitor.get_parameters()
     except Exception as ex:
