@@ -7,16 +7,17 @@ from argparse import SUPPRESS
 from contextlib import suppress
 from typing import Any, Callable, List, Optional, Set, Tuple, Type, Union
 
+from ._common import is_dataclass_like, is_subclass
 from .actions import _ActionConfigLoad
-from .optionals import get_doc_short_description
+from .optionals import get_doc_short_description, pydantic_support
 from .parameter_resolvers import (
     ParamData,
     get_parameter_origins,
     get_signature_parameters,
 )
 from .typehints import ActionTypeHint, LazyInitBaseClass, is_optional
-from .typing import is_final_class
-from .util import LoggerProperty, get_import_path, is_subclass, iter_to_set_str
+from .typing import register_pydantic_type
+from .util import LoggerProperty, get_import_path, iter_to_set_str
 
 __all__ = [
     'compose_dataclasses',
@@ -87,7 +88,7 @@ class SignatureArguments(LoggerProperty):
         if default:
             skip = skip or set()
             prefix = nested_key+'.' if nested_key else ''
-            defaults = default.lazy_get_init_data().as_dict()
+            defaults = default.lazy_get_init_args()
             if defaults:
                 defaults = {prefix+k: v for k, v in defaults.items() if k not in skip}
                 self.set_defaults(**defaults)  # type: ignore
@@ -311,7 +312,7 @@ class SignatureArguments(LoggerProperty):
         elif not as_positional:
             kwargs['required'] = True
         is_subclass_typehint = False
-        is_final_class_typehint = is_final_class(annotation)
+        is_dataclass_like_typehint = is_dataclass_like(annotation)
         dest = (nested_key+'.' if nested_key else '') + name
         args = [dest if is_required and as_positional else '--'+dest]
         if param.origin:
@@ -325,9 +326,9 @@ class SignatureArguments(LoggerProperty):
                 )
         if annotation in {str, int, float, bool} or \
            is_subclass(annotation, (str, int, float)) or \
-           is_final_class_typehint or \
-           is_pure_dataclass(annotation):
+           is_dataclass_like_typehint:
             kwargs['type'] = annotation
+            register_pydantic_type(annotation)
         elif annotation != inspect_empty:
             try:
                 is_subclass_typehint = ActionTypeHint.is_subclass_typehint(annotation, all_subtypes=False)
@@ -337,6 +338,8 @@ class SignatureArguments(LoggerProperty):
                     prefix = name + '.init_args.'
                     subclass_skip = {s[len(prefix):] for s in skip or [] if s.startswith(prefix)}
                     sub_add_kwargs['skip'] = subclass_skip
+                else:
+                    register_pydantic_type(annotation)
                 args = ActionTypeHint.prepare_add_argument(
                     args=args,
                     kwargs=kwargs,
@@ -353,7 +356,7 @@ class SignatureArguments(LoggerProperty):
                 'sub_configs': sub_configs,
                 'instantiate': instantiate,
             }
-            if is_final_class_typehint:
+            if is_dataclass_like_typehint:
                 kwargs.update(sub_add_kwargs)
             action = group.add_argument(*args, **kwargs)
             action.sub_add_kwargs = sub_add_kwargs
@@ -361,7 +364,12 @@ class SignatureArguments(LoggerProperty):
                 action.sub_add_kwargs['skip'] = subclass_skip
             added_args.append(dest)
         elif is_required and fail_untyped:
-            raise ValueError(f'Required parameter without a type for "{src}" parameter "{name}".')
+            msg = f'With fail_untyped=True, all mandatory parameters must have a supported type. Parameter "{name}" from "{src}" '
+            if isinstance(annotation, str):
+                msg += 'specifies the type as a string. Types as a string and `from __future__ import annotations` is currently not supported.'
+            else:
+                msg += 'does not specify a type.'
+            raise ValueError(msg)
 
 
     def add_dataclass_arguments(
@@ -370,6 +378,7 @@ class SignatureArguments(LoggerProperty):
         nested_key: str,
         default: Optional[Union[Type, dict]] = None,
         as_group: bool = True,
+        fail_untyped: bool = True,
         **kwargs
     ) -> List[str]:
         """Adds arguments from a dataclass based on its field types and docstrings.
@@ -379,6 +388,7 @@ class SignatureArguments(LoggerProperty):
             nested_key: Key for nested namespace.
             default: Value for defaults. Must be instance of or kwargs for theclass.
             as_group: Whether arguments should be added to a new argument group.
+            fail_untyped: Whether to raise exception if a required parameter does not have a type.
 
         Returns:
             The list of arguments added.
@@ -387,8 +397,8 @@ class SignatureArguments(LoggerProperty):
             ValueError: When not given a dataclass.
             ValueError: When default is not instance of or kwargs for theclass.
         """
-        if not is_pure_dataclass(theclass):
-            raise ValueError(f'Expected "theclass" argument to be a pure dataclass, given {theclass}')
+        if not is_dataclass_like(theclass):
+            raise ValueError(f'Expected "theclass" argument to be a dataclass-like, given {theclass}')
 
         doc_group = get_doc_short_description(theclass, logger=self.logger)
         for key in ['help', 'title']:
@@ -403,17 +413,19 @@ class SignatureArguments(LoggerProperty):
                     default = theclass(**default)
             if not isinstance(default, theclass):
                 raise ValueError(f'Expected "default" argument to be an instance of "{theclass.__name__}" or its kwargs dict, given {default}')
-            defaults = dataclasses.asdict(default)
+            defaults = dataclass_to_dict(default)
 
         added_args: List[str] = []
-        params = {p.name: p for p in get_signature_parameters(theclass, None, logger=self.logger)}
-        for field in dataclasses.fields(theclass):
+        param_kwargs = {k: v for k, v in kwargs.items() if k == 'sub_configs'}
+        for param in get_signature_parameters(theclass, None, logger=self.logger):
             self._add_signature_parameter(
                 group,
                 nested_key,
-                params[field.name],
+                param,
                 added_args,
-                default=defaults.get(field.name, inspect_empty),
+                fail_untyped=fail_untyped,
+                default=defaults.get(param.name, inspect_empty),
+                **param_kwargs,
             )
 
         return added_args
@@ -453,8 +465,8 @@ class SignatureArguments(LoggerProperty):
         Raises:
             ValueError: When given an invalid base class.
         """
-        if is_final_class(baseclass):
-            raise ValueError("Not allowed for classes that are final.")
+        if is_dataclass_like(baseclass):
+            raise ValueError("Not allowed for dataclass-like classes.")
         if type(baseclass) is not tuple:
             baseclass = (baseclass,)  # type: ignore
         if not all(inspect.isclass(c) for c in baseclass):
@@ -522,7 +534,6 @@ def group_instantiate_class(group, cfg):
         value = {}
         parent = cfg
         key = group.dest
-        assert '.' not in key
     parent[key] = group.group_class(**value)
 
 
@@ -536,15 +547,18 @@ def is_factory_class(value):
     return value.__class__ == dataclasses._HAS_DEFAULT_FACTORY_CLASS
 
 
-def is_pure_dataclass(value):
-    if not inspect.isclass(value):
-        return False
-    classes = [c for c in inspect.getmro(value) if c != object]
-    return all(dataclasses.is_dataclass(c) for c in classes)
+def dataclass_to_dict(value) -> dict:
+    if pydantic_support:
+        from pydantic import BaseModel  # pylint: disable=no-name-in-module
+        if isinstance(value, BaseModel):
+            return value.dict()
+    if isinstance(value, LazyInitBaseClass):
+        return value.lazy_get_init_data().as_dict()
+    return dataclasses.asdict(value)
 
 
 def compose_dataclasses(*args):
-    """Returns a pure dataclass inheriting all given dataclasses and properly handling __post_init__."""
+    """Returns a dataclass inheriting all given dataclasses and properly handling __post_init__."""
 
     @dataclasses.dataclass
     class ComposedDataclass(*args):
