@@ -4,6 +4,7 @@ import inspect
 import os
 import re
 import sys
+from argparse import ArgumentError
 from collections import abc, defaultdict
 from contextlib import contextmanager, suppress
 from contextvars import ContextVar
@@ -31,7 +32,7 @@ from typing import (
     Union,
 )
 
-from ._common import parent_parser, parser_context
+from ._common import is_dataclass_like, is_subclass, parent_parser, parser_context
 from .actions import (
     Action,
     ActionConfigFile,
@@ -54,22 +55,21 @@ from .optionals import (
     get_files_completer,
     typing_extensions_import,
 )
-from .typing import get_registered_type, is_final_class
+from .typing import get_registered_type, is_pydantic_type
 from .util import (
     ClassType,
     NestedArg,
     NoneType,
-    ParserError,
     Path,
     change_to_path_dir,
     get_import_path,
     get_typehint_origin,
     import_object,
     indent_text,
-    is_subclass,
     iter_to_set_str,
     object_path_serializer,
     parse_value_or_config,
+    read_stdin,
     warning,
 )
 
@@ -136,6 +136,7 @@ class ActionTypeHint(Action):
         """
         if typehint is not None:
             if not self.is_supported_typehint(typehint, full=True):
+                self.is_supported_typehint(typehint, full=True)
                 raise ValueError(f'Unsupported type hint {typehint}.')
             if get_typehint_origin(typehint) == Union:
                 subtype_supported = [
@@ -145,11 +146,7 @@ class ActionTypeHint(Action):
                 if sum(subtype_supported) < len(subtype_supported):
                     discard = {typehint.__args__[n] for n, s in enumerate(subtype_supported) if not s}
                     kwargs['logger'].debug(f'Discarding unsupported subtypes {discard} from {typehint}')
-                    orig_typehint = typehint  # deepcopy does not copy ForwardRef
-                    typehint = deepcopy(orig_typehint)
-                    typehint.__args__ = tuple(
-                        orig_typehint.__args__[n] for n, s in enumerate(subtype_supported) if s
-                    )
+                    typehint = Union[tuple(t for t, s in zip(typehint.__args__, subtype_supported) if s)]  # type: ignore
             self._typehint = typehint
             self._enable_path = False if is_optional(typehint, Path) else enable_path
         elif '_typehint' not in kwargs:
@@ -201,6 +198,7 @@ class ActionTypeHint(Action):
             get_typehint_origin(typehint) in root_types or \
             get_registered_type(typehint) is not None or \
             is_subclass(typehint, Enum) or \
+            is_dataclass_like(typehint) or \
             ActionTypeHint.is_subclass_typehint(typehint)
         if full and supported:
             typehint_origin = get_typehint_origin(typehint) or typehint
@@ -236,6 +234,8 @@ class ActionTypeHint(Action):
         return inspect.isclass(typehint) and \
             typehint not in leaf_or_root_types and \
             not get_registered_type(typehint) and \
+            not is_pydantic_type(typehint) and \
+            not is_dataclass_like(typehint) and \
             typehint_origin is None and \
             not is_subclass(typehint, (Path, Enum))
 
@@ -397,7 +397,10 @@ class ActionTypeHint(Action):
         cfg, val, opt_str = args[1:]
         if not (self.nargs == '?' and val is None):
             if isinstance(opt_str, str) and opt_str.startswith(f'--{self.dest}.'):
-                sub_opt = opt_str[len(f'--{self.dest}.'):]
+                if opt_str.startswith(f'--{self.dest}.init_args.'):
+                    sub_opt = opt_str[len(f'--{self.dest}.init_args.'):]
+                else:
+                    sub_opt = opt_str[len(f'--{self.dest}.'):]
                 val = NestedArg(key=sub_opt, val=val)
             append = opt_str == f'--{self.dest}+'
             val = self._check_type(val, append=append, cfg=cfg)
@@ -435,6 +438,7 @@ class ActionTypeHint(Action):
                     'sub_add_kwargs': getattr(self, 'sub_add_kwargs', {}),
                     'prev_val': prev_val,
                     'append': append,
+                    'enable_path': enable_path,
                 }
                 try:
                     with change_to_path_dir(config_path):
@@ -482,7 +486,7 @@ class ActionTypeHint(Action):
         if skip_args:
             kwargs.setdefault('skip', set()).add(skip_args)
         parser = parent_parser.get()
-        parser = type(parser)(error_handler=None, logger=parser.logger)
+        parser = type(parser)(exit_on_error=False, logger=parser.logger)
         remove_actions(parser, (ActionConfigFile, _ActionPrintConfig))
         parser.add_class_arguments(val_class, **kwargs)
         return parser
@@ -534,7 +538,8 @@ def raise_unexpected_value(message: str, val: Any = inspect._empty, exception: O
 
 
 def raise_union_unexpected_value(uniontype, val: Any, exceptions: List[Exception]) -> NoReturn:
-    errors = indent_text('- ' + '\n- '.join(map(str, exceptions)))
+    str_exceptions = [indent_text(str(e), first_line=False) for e in exceptions]
+    errors = indent_text('- ' + '\n- '.join(str_exceptions))
     errors = errors.replace(f'. Got value: {val}', '').replace(f' {val} ', ' ')
     subtypes = uniontype.__args__
     raise ValueError(
@@ -543,13 +548,22 @@ def raise_union_unexpected_value(uniontype, val: Any, exceptions: List[Exception
     ) from exceptions[0]
 
 
-def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, prev_val=None, append=False, sub_add_kwargs=None):
-
+def adapt_typehints(
+    val,
+    typehint,
+    serialize=False,
+    instantiate_classes=False,
+    prev_val=None,
+    append=False,
+    enable_path=False,
+    sub_add_kwargs=None,
+):
     adapt_kwargs = {
         'serialize': serialize,
         'instantiate_classes': instantiate_classes,
         'prev_val': prev_val,
         'append': append,
+        'enable_path': enable_path,
         'sub_add_kwargs': sub_add_kwargs or {},
     }
     subtypehints = getattr(typehint, '__args__', None)
@@ -587,10 +601,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
     elif get_registered_type(typehint):
         registered_type = get_registered_type(typehint)
         if serialize:
-            if registered_type.is_value_of_type(val):
-                val = registered_type.serializer(val)
-            else:
-                registered_type.deserializer(val)
+            val = registered_type.serializer(val)
         elif not serialize and not registered_type.is_value_of_type(val):
             val = registered_type.deserializer(val)
 
@@ -599,14 +610,11 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
         if serialize:
             if isinstance(val, typehint):
                 val = val.name
-            else:
-                if val not in typehint.__members__:
-                    raise_unexpected_value(f'Expected a member of {typehint}', val)
-        elif not serialize and not isinstance(val, typehint):
+        elif not isinstance(val, typehint):
             try:
                 val = typehint[val]
             except KeyError as ex:
-                raise_unexpected_value(f'Expected a member of {typehint}', val, ex)
+                raise_unexpected_value(f'Expected a member of {typehint}: {iter_to_set_str(typehint.__members__)}', val, ex)
 
     # Type
     elif typehint in {Type, type} or typehint_origin in {Type, type}:
@@ -665,6 +673,14 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
             val_is_list = isinstance(val, list)
             val = prev_val + (val if val_is_list else [val])
             prev_val = prev_val + [None] * (len(val)-len(prev_val) if val_is_list else 1)
+        if enable_path and type(val) is str:
+            if val == '-':
+                val = read_stdin().splitlines()
+            else:
+                with suppress(TypeError):
+                    from .optionals import get_config_read_mode
+                    list_path = Path(val, mode=get_config_read_mode())
+                    val = list_path.get_content().splitlines()
         if isinstance(val, NestedArg) and subtypehints is not None:
             val = (prev_val[:-1] if isinstance(prev_val, list) else []) + [val]
         elif isinstance(val, Iterable) and not isinstance(val, (list, str)) and type(val) not in mapping_origin_types:
@@ -707,7 +723,8 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
     elif typehint_origin in callable_origin_types or typehint in callable_origin_types:
         if serialize:
             if is_subclass_spec(val):
-                val = adapt_class_type(val, True, False, sub_add_kwargs)
+                val, _, num_partial_args = adapt_partial_callable_class(typehint, val)
+                val = adapt_class_type(val, True, False, sub_add_kwargs, skip_args=num_partial_args)
             else:
                 val = object_path_serializer(val)
         else:
@@ -735,9 +752,22 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
                         raise ImportError(f'Expected {val["class_path"]!r} to be a class that instantiates into callable or a subclass of {partial_classes}')
                     val['class_path'] = get_import_path(val_class)
                     val = adapt_class_type(val, False, instantiate_classes, sub_add_kwargs, skip_args=num_partial_args)
-            except (ImportError, AttributeError, ParserError) as ex:
+            except (ImportError, AttributeError, ArgumentError) as ex:
                 raise_unexpected_value(f'Type {typehint} expects a function or a callable class: {ex}', val, ex)
 
+    # Dataclass-like
+    elif is_dataclass_like(typehint):
+        parser = ActionTypeHint.get_class_parser(typehint, sub_add_kwargs)
+        if instantiate_classes:
+            init_args = parser.instantiate_classes(val)
+            return typehint(**init_args)
+        if serialize:
+            val = load_value(parser.dump(val, **dump_kwargs.get()))
+        elif isinstance(val, (dict, Namespace)):
+            val = parser.parse_object(val, defaults=sub_defaults.get())
+        else:
+            raise_unexpected_value(f'Type {typehint} expects a dict or Namespace', val)
+
     # Subclass
     elif not hasattr(typehint, '__origin__') and inspect.isclass(typehint):
         if isinstance(val, typehint):
@@ -751,7 +781,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
         val = subclass_spec_as_namespace(val, prev_val)
         if not is_subclass_spec(val):
             raise_unexpected_value(
-                f'Not a valid {typehint.__name__}. Got value: {val_input}\n'
+                f'Not a valid subclass of {typehint.__name__}. Got value: {val_input}\n'
                 'Subclass types expect one of:\n'
                 '- a class path (str)\n'
                 '- a dict with class_path entry\n'
@@ -764,10 +794,10 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
                 raise_unexpected_value(f'Import path {val["class_path"]} does not correspond to a subclass of {typehint}')
             val['class_path'] = get_import_path(val_class)
             val = adapt_class_type(val, serialize, instantiate_classes, sub_add_kwargs, prev_val=prev_val)
-        except (ImportError, AttributeError, AssertionError, ParserError) as ex:
+        except (ImportError, AttributeError, AssertionError, ArgumentError) as ex:
             class_path = val if isinstance(val, str) else val['class_path']
-            error = indent_text(f'\n- {ex}')
-            raise_unexpected_value(f'Problem with given class_path {class_path!r}:{error}', exception=ex)
+            error = indent_text(str(ex))
+            raise_unexpected_value(f'Problem with given class_path {class_path!r}:\n{error}', exception=ex)
 
     return val
 
@@ -790,11 +820,7 @@ def subclass_spec_as_namespace(val, prev_val=None):
         if '.' not in key:
             root_key = key
         else:
-            if key.startswith('init_args.'):
-                root_key = 'init_args'
-                key = key[len('init_args.'):]
-                val = Namespace({key: val})
-            elif key.startswith('dict_kwargs.'):
+            if key.startswith('dict_kwargs.'):
                 root_key = 'dict_kwargs'
                 key = key[len('dict_kwargs.'):]
                 val = {key: val}
@@ -923,6 +949,7 @@ def discard_init_args_on_class_path_change(parser_or_action, prev_val, value):
             sub_add_kwargs = getattr(parser_or_action, 'sub_add_kwargs', {})
             parser = ActionTypeHint.get_class_parser(value['class_path'], sub_add_kwargs)
         del_args = {}
+        prev_val = subclass_spec_as_namespace(prev_val)
         for key, val in list(prev_val.init_args.__dict__.items()):
             action = _find_action(parser, key)
             if action:
@@ -1124,11 +1151,11 @@ def callable_instances(cls: Type):
 def check_lazy_kwargs(class_type: Type, lazy_kwargs: dict):
     if lazy_kwargs:
         from .core import ArgumentParser
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_class_arguments(class_type)
         try:
             parser.parse_object(lazy_kwargs)
-        except ParserError as ex:
+        except ArgumentError as ex:
             raise ValueError(str(ex)) from ex
 
 
@@ -1162,9 +1189,12 @@ class LazyInitBaseClass:
         self._lazy_init()
         return getattr(self, method_name)(*args, **kwargs)
 
+    def lazy_get_init_args(self) -> Namespace:
+        return Namespace(self._lazy_kwargs)
+
     def lazy_get_init_data(self):
-        init_args = Namespace(self._lazy_kwargs)
-        if is_final_class(self._lazy_class_type):
+        init_args = self.lazy_get_init_args()
+        if is_dataclass_like(self._lazy_class_type):
             return init_args
         init = Namespace(class_path=get_import_path(self._lazy_class_type))
         if len(self._lazy_kwargs) > 0:
