@@ -1,13 +1,16 @@
 """Collection of types and type generators."""
 
+import inspect
 import operator
 import os
 import pathlib
 import re
+from enum import Enum
 from typing import Any, Callable, Dict, List, Optional, Pattern, Tuple, Type, Union
 
-from .optionals import final
-from .util import Path, get_import_path, import_object
+from ._common import is_final_class, is_subclass
+from .optionals import final, pydantic_support
+from .util import Path, get_import_path, get_private_kwargs, import_object
 
 __all__ = [
     'final',
@@ -196,14 +199,13 @@ def _is_path_type(value, type_class):
 def path_type(
     mode: str,
     docstring: Optional[str] = None,
-    skip_check: bool = False,
+    **kwargs
 ) -> type:
     """Creates or returns an already registered path type class.
 
     Args:
         mode: The required type and access permissions among [fdrwxcuFDRWX].
         docstring: Docstring for the type class.
-        skip_check: Whether to skip path checks.
 
     Returns:
         The created or retrieved type class.
@@ -211,7 +213,11 @@ def path_type(
     Path._check_mode(mode)
     name = 'Path_'+mode
     key_name = 'path '+''.join(sorted(mode))
+
+    skip_check = get_private_kwargs(kwargs, skip_check=False)
     if skip_check:
+        from .deprecated import path_skip_check_deprecation
+        path_skip_check_deprecation()
         name += '_skip_check'
         key_name += ' skip_check'
 
@@ -324,11 +330,6 @@ def add_type(type_class: Type, uniqueness_key: Optional[Tuple], type_check: Opti
     register_type(type_class, type_class._type, **kwargs)  # type: ignore
 
 
-def is_final_class(cls):
-    """Checks whether a class is final, i.e. decorated with ``final``."""
-    return getattr(cls, '__final__', False)
-
-
 _fail_already_registered = False
 
 PositiveInt        = restricted_number_type('PositiveInt',        int, ('>', 0),
@@ -401,4 +402,52 @@ register_type_on_first_use('builtins.bytes', serializer=bytes_serializer, deseri
 register_type_on_first_use('builtins.bytearray', serializer=bytes_serializer, deserializer=bytearray_deserializer)
 
 
+pydantic_types: Tuple[type, ...] = tuple()
+if pydantic_support:
+    import pydantic
+    for module in [pydantic.types, pydantic.networks]:
+        pydantic_types += tuple(
+            v for k, v in vars(module).items()
+            if inspect.isclass(v) and k in module.__all__ and not issubclass(v, Enum)
+        )
+
+
+def pydantic_deserializer(type_class):
+    from pydantic import create_model  # pylint: disable=no-name-in-module
+    pydantic_model = create_model('pydantic_model', pydantic_field=(type_class, ...))
+
+    def deserialize(value):
+        return pydantic_model(pydantic_field=value).pydantic_field
+
+    return deserialize
+
+
+def pydantic_serializer(type_class):
+    serializer = str
+    for base in [int, float, bool, list, dict, (set, list)]:
+        if not isinstance(base, tuple):
+            base = (base, base)
+        if issubclass(type_class, base[0]):
+            serializer = base[1]
+            break
+    return serializer
+
+
+def is_pydantic_type(type_class):
+    return pydantic_support and is_subclass(type_class, pydantic_types)
+
+
+def register_pydantic_type(type_class):
+    if not is_pydantic_type(type_class):
+        return
+    if not get_registered_type(type_class):
+        from pydantic import ValidationError
+        register_type(
+            type_class=type_class,
+            serializer=pydantic_serializer(type_class),
+            deserializer=pydantic_deserializer(type_class),
+            deserializer_exceptions=ValidationError,
+        )
+
+
 del _fail_already_registered
