@@ -2,17 +2,15 @@
 
 import json
 import os
-import pathlib
 import unittest
 from io import StringIO
 
 from jsonargparse import (
     ActionConfigFile,
     ActionParser,
-    ActionPathList,
     ActionYesNo,
+    ArgumentError,
     ArgumentParser,
-    ParserError,
     strip_meta,
 )
 from jsonargparse_tests.base import TempDirTestCase
@@ -35,7 +33,7 @@ class SimpleActionsTests(unittest.TestCase):
         self.assertEqual(True,  parser.parse_args(['--bools.def_false=yes']).bools.def_false)
         self.assertEqual(False, parser.parse_args(['--bools.def_false=no']).bools.def_false)
         self.assertEqual(True,  parser.parse_args(['--no_bools.def_true=no']).bools.def_true)
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--bools.def_true nope']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--bools.def_true nope']))
 
         parser = ArgumentParser()
         parser.add_argument('--val', action=ActionYesNo)
@@ -64,69 +62,14 @@ class SimpleActionsTests(unittest.TestCase):
 
 
     def test_ActionYesNo_old_bool(self):
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_argument('--val', nargs=1, action=ActionYesNo(no_prefix=None))
         self.assertEqual(False, parser.get_defaults().val)
         self.assertEqual(True,  parser.parse_args(['--val', 'true']).val)
         self.assertEqual(True,  parser.parse_args(['--val', 'yes']).val)
         self.assertEqual(False, parser.parse_args(['--val', 'false']).val)
         self.assertEqual(False, parser.parse_args(['--val', 'no']).val)
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--val', '1']))
-
-
-class ActionPathTests(TempDirTestCase):
-
-    def test_ActionPathList(self):
-        tmpdir = os.path.join(self.tmpdir, 'subdir')
-        os.mkdir(tmpdir)
-        pathlib.Path(os.path.join(tmpdir, 'file1')).touch()
-        pathlib.Path(os.path.join(tmpdir, 'file2')).touch()
-        pathlib.Path(os.path.join(tmpdir, 'file3')).touch()
-        pathlib.Path(os.path.join(tmpdir, 'file4')).touch()
-        pathlib.Path(os.path.join(tmpdir, 'file5')).touch()
-        list_file = os.path.join(tmpdir, 'files.lst')
-        list_file2 = os.path.join(tmpdir, 'files2.lst')
-        list_file3 = os.path.join(tmpdir, 'files3.lst')
-        list_file4 = os.path.join(tmpdir, 'files4.lst')
-        with open(list_file, 'w') as output_file:
-            output_file.write('file1\nfile2\nfile3\nfile4\n')
-        with open(list_file2, 'w') as output_file:
-            output_file.write('file5\n')
-        pathlib.Path(list_file3).touch()
-        with open(list_file4, 'w') as output_file:
-            output_file.write('file1\nfile2\nfile6\n')
-
-        parser = ArgumentParser(prog='app', error_handler=None)
-        parser.add_argument('--list',
-            nargs='+',
-            action=ActionPathList(mode='fr', rel='list'))
-        parser.add_argument('--list_cwd',
-            action=ActionPathList(mode='fr', rel='cwd'))
-
-        cfg = parser.parse_args(['--list', list_file])
-        self.assertEqual(4, len(cfg.list))
-        self.assertEqual(['file1', 'file2', 'file3', 'file4'], [str(x) for x in cfg.list])
-
-        cfg = parser.parse_args(['--list', list_file, list_file2])
-        self.assertEqual(5, len(cfg.list))
-        self.assertEqual(['file1', 'file2', 'file3', 'file4', 'file5'], [str(x) for x in cfg.list])
-
-        self.assertEqual(0, len(parser.parse_args(['--list', list_file3]).list))
-
-        cwd = os.getcwd()
-        os.chdir(tmpdir)
-        cfg = parser.parse_args(['--list_cwd', list_file])
-        self.assertEqual(4, len(cfg.list_cwd))
-        self.assertEqual(['file1', 'file2', 'file3', 'file4'], [str(x) for x in cfg.list_cwd])
-        os.chdir(cwd)
-
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--list']))
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--list', list_file4]))
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--list', 'no-such-file']))
-
-        self.assertRaises(ValueError, lambda: parser.add_argument('--op1', action=ActionPathList))
-        self.assertRaises(ValueError, lambda: parser.add_argument('--op2', action=ActionPathList(mode='fr'), nargs='*'))
-        self.assertRaises(ValueError, lambda: parser.add_argument('--op3', action=ActionPathList(mode='fr', rel='.')))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--val', '1']))
 
 
 class ActionParserTests(TempDirTestCase):
@@ -142,7 +85,7 @@ class ActionParserTests(TempDirTestCase):
         parser_lv2.add_argument('--inner3',
             action=ActionParser(parser=parser_lv3))
 
-        parser = ArgumentParser(prog='lv1', default_env=True, error_handler=None)
+        parser = ArgumentParser(prog='lv1', default_env=True, exit_on_error=False)
         parser.add_argument('--opt1',
             default='opt1_def')
         parser.add_argument('--inner2',
@@ -226,22 +169,22 @@ class ActionParserTests(TempDirTestCase):
     def test_ActionParser_required(self):
         p1 = ArgumentParser()
         p1.add_argument('--op1', required=True)
-        p2 = ArgumentParser(error_handler=None)
+        p2 = ArgumentParser(exit_on_error=False)
         p2.add_argument('--op2', action=ActionParser(parser=p1))
         p2.parse_args(['--op2.op1=1'])
-        self.assertRaises(ParserError, lambda: p2.parse_args([]))
+        self.assertRaises(ArgumentError, lambda: p2.parse_args([]))
 
 
     def test_ActionParser_failures(self):
         parser_lv2 = ArgumentParser()
         parser_lv2.add_argument('--op')
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_argument('--inner', action=ActionParser(parser=parser_lv2))
         self.assertRaises(Exception, lambda: parser.add_argument('--bad', action=ActionParser))
         self.assertRaises(ValueError, lambda: parser.add_argument('--bad', action=ActionParser(parser=parser)))
         self.assertRaises(ValueError, lambda: parser.add_argument('--bad', type=str, action=ActionParser(ArgumentParser())))
         self.assertRaises(ValueError, lambda: parser.add_argument('-b', '--bad', action=ActionParser(ArgumentParser())))
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--inner=1']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--inner=1']))
         self.assertRaises(ValueError, lambda: ActionParser())
         self.assertRaises(ValueError, lambda: ActionParser(parser=object))
 
@@ -249,21 +192,21 @@ class ActionParserTests(TempDirTestCase):
     def test_ActionParser_conflict(self):
         parser_lv2 = ArgumentParser()
         parser_lv2.add_argument('--op')
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_argument('--inner.op')
         self.assertRaises(ValueError, lambda: parser.add_argument('--inner', action=ActionParser(parser_lv2)))
 
 
     def test_ActionParser_nested_dash_names(self):
-        p1 = ArgumentParser(error_handler=None)
+        p1 = ArgumentParser(exit_on_error=False)
         p1.add_argument('--op1-like')
 
-        p2 = ArgumentParser(error_handler=None)
+        p2 = ArgumentParser(exit_on_error=False)
         p2.add_argument('--op2-like', action=ActionParser(parser=p1))
 
         self.assertEqual(p2.parse_args(['--op2-like.op1-like=a']).op2_like.op1_like, 'a')
 
-        p3 = ArgumentParser(error_handler=None)
+        p3 = ArgumentParser(exit_on_error=False)
         p3.add_argument('--op3', action=ActionParser(parser=p2))
 
         self.assertEqual(p3.parse_args(['--op3.op2-like.op1-like=b']).op3.op2_like.op1_like, 'b')
