@@ -2,7 +2,6 @@
 
 import calendar
 import inspect
-import logging
 import unittest
 import xml.dom
 from calendar import Calendar
@@ -10,12 +9,14 @@ from contextlib import contextmanager
 from random import shuffle
 from typing import Any, Callable, Dict, List
 from unittest.mock import patch
-from jsonargparse import class_from_function, Namespace
+
+from jsonargparse import Namespace, class_from_function
 from jsonargparse.optionals import docstring_parser_support
-from jsonargparse.parameter_resolvers import (
-    get_signature_parameters as get_params,
-    is_lambda,
-)
+from jsonargparse.parameter_resolvers import get_signature_parameters as get_params
+from jsonargparse.parameter_resolvers import is_lambda
+from jsonargparse_tests.base import get_debug_level_logger
+
+logger = get_debug_level_logger(__name__)
 
 
 class ClassA:
@@ -420,10 +421,6 @@ def assert_params(self, params, expected, origins={}):
     self.assertEqual(param_origins, origins)
 
 
-logger = logging.getLogger('test_parameter_resolvers')
-logger.level = logging.DEBUG
-
-
 class GetClassParametersTests(unittest.TestCase):
 
     def test_get_params_class_no_inheritance_unused_kwargs(self):
