@@ -1,6 +1,5 @@
 #!/usr/bin/env python3
 
-import dataclasses
 import json
 import os
 import textwrap
@@ -17,16 +16,15 @@ import yaml
 
 from jsonargparse import (
     ActionConfigFile,
+    ArgumentError,
     ArgumentParser,
     Namespace,
-    ParserError,
     class_from_function,
-    compose_dataclasses,
     lazy_instance,
     strip_meta,
 )
 from jsonargparse.actions import _find_action
-from jsonargparse.optionals import docstring_parser_support, set_docstring_parse_options
+from jsonargparse.optionals import docstring_parser_support
 from jsonargparse.typing import OpenUnitInterval, PositiveFloat, PositiveInt, final
 from jsonargparse_tests.base import TempDirTestCase, mock_module, suppress_stderr
 
@@ -92,7 +90,7 @@ class SignaturesTests(unittest.TestCase):
                 super().__init__(None, c3_a4, **kwargs)
 
         ## Test without nesting ##
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_class_arguments(Class3)
 
         self.assertRaises(ValueError, lambda: parser.add_class_arguments('Class3'))
@@ -124,7 +122,7 @@ class SignaturesTests(unittest.TestCase):
         self.assertEqual(('3', 3, 3.0), parser.parse_args(['--c3_a0=0', '--c3_a7=["3", 3, 3.0]']).c3_a7)
         self.assertEqual('a', Class3(**cfg.as_dict())())
 
-        self.assertRaises(ParserError, lambda: parser.parse_args([]))  # c3_a0 is required
+        self.assertRaises(ArgumentError, lambda: parser.parse_args([]))  # c3_a0 is required
 
         if docstring_parser_support:
             self.assertEqual('Class3 short description', parser.groups['Class3'].title)
@@ -155,11 +153,22 @@ class SignaturesTests(unittest.TestCase):
         self.assertRaises(ValueError, lambda: parser.add_class_arguments(Class2))
 
 
+    def test_add_class_with_default(self):
+        class Class:
+            def __init__(self, p1: int = 1, p2: str = '-'):
+                pass
+
+        parser = ArgumentParser()
+        parser.add_class_arguments(Class, 'cls', default=lazy_instance(Class, p1=2))
+        defaults = parser.get_defaults()
+        self.assertEqual(defaults, Namespace(cls=Namespace(p1=2, p2='-')))
+
+
     def test_add_class_without_args(self):
         class NoArgs:
             pass
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_argument('--cfg', action=ActionConfigFile)
         parser.add_class_arguments(NoArgs, 'noargs')
 
@@ -174,7 +183,7 @@ class SignaturesTests(unittest.TestCase):
 
         with mock_module(NoArgs) as module:
             config = {'noargs': {'class_path': f'{module}.NoArgs'}}
-            with self.assertRaises(ParserError):
+            with self.assertRaises(ArgumentError):
                 parser.parse_args([f'--cfg={config}'])
 
 
@@ -187,11 +196,31 @@ class SignaturesTests(unittest.TestCase):
         self.assertEqual([], parser.add_class_arguments(NoValidArgs))
 
 
+    def test_instantiate_nested_class_without_args(self):
+        class CallbackWithArgs:
+            def __init__(self, p1: int):
+                self.p1 = p1
+
+        class CallbackWithoutArgs:
+            pass
+
+        parser = ArgumentParser()
+        parser.add_class_arguments(CallbackWithArgs, 'callbacks.first')
+        parser.add_class_arguments(CallbackWithoutArgs, 'callbacks.second')
+
+        cfg = parser.parse_args(['--callbacks.first.p1=2'])
+        self.assertEqual(cfg.callbacks.first, Namespace(p1=2))
+        self.assertNotIn('callbacks.second', cfg)
+        init = parser.instantiate_classes(cfg)
+        self.assertIsInstance(init.callbacks.first, CallbackWithArgs)
+        self.assertIsInstance(init.callbacks.second, CallbackWithoutArgs)
+
+
     def test_add_function_with_dict_int_keys_arg(self):
         def func(a1: Union[int, Dict[int, int]] = 1):
             pass
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_function_arguments(func)
         parser.get_defaults()
         self.assertEqual({2: 7, 4: 9}, parser.parse_args(['--a1={"2": 7, "4": 9}']).a1)
@@ -206,7 +235,7 @@ class SignaturesTests(unittest.TestCase):
                 obj.a2 = a2  # type: ignore
                 return obj
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_class_arguments(ClassA, 'a')
 
         cfg = parser.parse_args(['--a.a1=4'])
@@ -218,7 +247,7 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, n: int, m: float):
                 pass
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_class_arguments(Model, 'model')
         parser.add_argument('--config', action=ActionConfigFile)
 
@@ -228,7 +257,7 @@ class SignaturesTests(unittest.TestCase):
             ['--model.m=0.1'],
             ['--model.n=x', '--model.m=0.1'],
         ]:
-            with self.assertRaises(ParserError):
+            with self.assertRaises(ArgumentError):
                 parser.parse_args(args)
 
         out = StringIO()
@@ -388,7 +417,7 @@ class SignaturesTests(unittest.TestCase):
 
 
     def test_add_subclass_arguments(self):
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_subclass_arguments(Calendar, 'cal')
 
         cal = {'class_path': 'calendar.Calendar', 'init_args': {'firstweekday': 1}}
@@ -407,10 +436,10 @@ class SignaturesTests(unittest.TestCase):
         cfg = parser.parse_args(['--cal.class_path=calendar.Calendar', '--cal.init_args.firstweekday=4', '--cal.class_path=calendar.Calendar'])
         self.assertEqual(cfg['cal'].as_dict(), cal)
 
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--cal={"class_path":"not.exist.Class"}']))
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--cal={"class_path":"calendar.January"}']))
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--cal.help=calendar.January']))
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--cal.help=calendar.does_not_exist']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--cal={"class_path":"not.exist.Class"}']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--cal={"class_path":"calendar.January"}']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--cal.help=calendar.January']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--cal.help=calendar.does_not_exist']))
         self.assertRaises(ValueError, lambda: parser.add_subclass_arguments(January, 'jan'))
 
         out = StringIO()
@@ -478,13 +507,13 @@ class SignaturesTests(unittest.TestCase):
             self.assertIn('--cal.init_args.firstweekday', out.getvalue())
             self.assertNotIn('param', out.getvalue())
 
-            parser = ArgumentParser(error_handler=None)
+            parser = ArgumentParser(exit_on_error=False)
             parser.add_subclass_arguments(Calendar, 'cal')
             self.assertRaises(ValueError, lambda: parser.parse_args(args))
 
 
     def test_add_subclass_discard_init_args(self):
-        parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
+        parser = ArgumentParser(exit_on_error=False, logger={'level': 'DEBUG'})
         parser.add_subclass_arguments(Calendar, 'cal')
 
         class CalA(Calendar):
@@ -525,7 +554,7 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, c: ChildBase):
                 pass
 
-        parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
+        parser = ArgumentParser(exit_on_error=False, logger={'level': 'DEBUG'})
         with mock_module(Parent, ChildBase, A, B) as module, self.assertLogs(logger=parser.logger, level='DEBUG') as log:
             parser.add_subclass_arguments(Parent, 'p')
             cfg = parser.parse_args([
@@ -550,7 +579,7 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, cal: Union[Calendar, bool] = lazy_instance(MyCalendar, param=1)):
                 self.cal = cal
 
-        parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
+        parser = ArgumentParser(exit_on_error=False, logger={'level': 'DEBUG'})
         with mock_module(MyCalendar), self.assertLogs(logger=parser.logger, level='DEBUG') as log:
             parser.add_class_arguments(Main, 'main')
             parser.parse_args(['--main.cal=Calendar'])
@@ -558,12 +587,12 @@ class SignaturesTests(unittest.TestCase):
 
 
     def test_add_subclass_init_args_without_class_path(self):
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_subclass_arguments(Calendar, 'cal1')
         parser.add_subclass_arguments(Calendar, 'cal2', default=lazy_instance(Calendar))
         parser.add_subclass_arguments(Calendar, 'cal3', default=lazy_instance(Calendar, firstweekday=2))
 
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--cal1.init_args.firstweekday=4']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--cal1.init_args.firstweekday=4']))
         cfg = parser.parse_args(['--cal2.init_args.firstweekday=4', '--cal3.init_args.firstweekday=5'])
         self.assertEqual(cfg.cal2.init_args, Namespace(firstweekday=4))
         self.assertEqual(cfg.cal3.init_args, Namespace(firstweekday=5))
@@ -580,7 +609,7 @@ class SignaturesTests(unittest.TestCase):
                 self.image_size = image_size
 
         with mock_module(ModelBaseClass, ModelClass) as module:
-            parser = ArgumentParser(error_handler=None)
+            parser = ArgumentParser(exit_on_error=False)
             parser.add_subclass_arguments(ModelBaseClass, 'model')
             parser.add_argument('--config', action=ActionConfigFile)
 
@@ -604,7 +633,7 @@ class SignaturesTests(unittest.TestCase):
 
 
     def test_add_subclass_init_args_in_subcommand(self):
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         subcommands = parser.add_subcommands()
         subparser = ArgumentParser()
         subparser.add_subclass_arguments(Calendar, 'cal', default=lazy_instance(Calendar))
@@ -630,7 +659,7 @@ class SignaturesTests(unittest.TestCase):
             class_path_a = f'{module}.ClassA'
             class_path_b = f'{module}.ClassB'
 
-            parser = ArgumentParser(error_handler=None)
+            parser = ArgumentParser(exit_on_error=False)
             parser.add_subclass_arguments((ClassA, ClassB), 'c')
 
             cfg = parser.parse_args(['--c={"class_path": "'+class_path_a+'", "init_args": {"a1": -1}}'])
@@ -651,17 +680,17 @@ class SignaturesTests(unittest.TestCase):
 
 
     def test_required_group(self):
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         self.assertRaises(ValueError, lambda: parser.add_subclass_arguments(Calendar, None, required=True))
         parser.add_subclass_arguments(Calendar, 'cal', required=True)
-        self.assertRaises(ParserError, lambda: parser.parse_args([]))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args([]))
         out = StringIO()
         parser.print_help(out)
         self.assertIn('[-h] [--cal.help CLASS_PATH_OR_NAME] --cal ', out.getvalue())
 
 
     def test_not_required_group(self):
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_subclass_arguments(Calendar, 'cal', required=False)
         cfg = parser.parse_args([])
         self.assertEqual(cfg, Namespace())
@@ -674,7 +703,7 @@ class SignaturesTests(unittest.TestCase):
         def func(a1: None):
             return a1
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         self.assertRaises(ValueError, lambda: parser.add_function_arguments(func))
 
 
@@ -688,10 +717,10 @@ class SignaturesTests(unittest.TestCase):
         def func(a1: Optional[MyEnum] = None):
             return a1
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_function_arguments(func)
         self.assertEqual(MyEnum.B, parser.parse_args(['--a1=B']).a1)
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--a1=D']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--a1=D']))
 
         help_str = StringIO()
         parser.print_help(help_str)
@@ -704,7 +733,7 @@ class SignaturesTests(unittest.TestCase):
         def func2(a1: Optional[MyEnum2] = None):
             return a1
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_function_arguments(func2)
         self.assertEqual(MyEnum2.B, parser.parse_args(['--a1=B']).a1)
         self.assertEqual('B', parser.parse_args(['--a1=B']).a1)
@@ -719,7 +748,7 @@ class SignaturesTests(unittest.TestCase):
         def func(a1: Any = MyEnum.B):
             return a1
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_function_arguments(func)
         cfg = parser.parse_args([])
         self.assertEqual('a1: B\n', parser.dump(cfg))
@@ -761,7 +790,7 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, c1: Class1, c2: int = 5, c3: float = 6.7):
                 pass
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_class_arguments(Class2, skip={'c1.init_args.a2', 'c2'})
 
         with mock_module(Class1) as module:
@@ -782,7 +811,7 @@ class SignaturesTests(unittest.TestCase):
                 self.b1 = b1
                 self.b2 = b2
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_subclass_arguments(ClassA, 'c', skip={'a1', 'b2'})
 
         with mock_module(ClassA, ClassB) as module:
@@ -808,7 +837,7 @@ class SignaturesTests(unittest.TestCase):
                 self.b1 = b1
                 self.b2 = b2
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_class_arguments(ClassB, 'b')
 
         self.assertEqual(parser.get_defaults().b.b2, Namespace(a1=1, a2=-3.2))
@@ -819,8 +848,8 @@ class SignaturesTests(unittest.TestCase):
         self.assertIsInstance(cfg['b'], ClassB)
         self.assertIsInstance(cfg['b'].b2, ClassA)
 
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--b.b2={"bad": "value"}']))
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--b.b2="bad"']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--b.b2={"bad": "value"}']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--b.b2="bad"']))
         self.assertRaises(ValueError, lambda: parser.add_subclass_arguments(ClassA, 'a'))
         self.assertRaises(ValueError, lambda: parser.add_class_arguments(ClassA, 'a', default=ClassA()))
 
@@ -831,18 +860,18 @@ class SignaturesTests(unittest.TestCase):
                  a2: Optional[Union[PositiveInt, OpenUnitInterval]] = 0.5):
             return a1, a2
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_function_arguments(func)
 
         self.assertEqual(1.0, parser.parse_args(['--a1=1']).a1)
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--a1=-1']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--a1=-1']))
 
         self.assertEqual(0.7, parser.parse_args(['--a2=0.7']).a2)
         self.assertEqual(5, parser.parse_args(['--a2=5']).a2)
         self.assertEqual(None, parser.parse_args(['--a2=null']).a2)
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--a2=0']))
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--a2=1.5']))
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--a2=-1']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--a2=0']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--a2=1.5']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--a2=-1']))
 
 
     def test_dict_int_str_type(self):
@@ -850,12 +879,12 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, d: Dict[int, str]):
                 self.d = d
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_class_arguments(Foo)
         parser.add_argument('--config', action=ActionConfigFile)
         cfg = {'d': {1: 'val1', 2: 'val2'}}
         self.assertEqual(cfg['d'], parser.parse_args(['--config', str(cfg)]).d)
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--config={"d": {"a": "b"}}']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--config={"d": {"a": "b"}}']))
 
 
     def test_logger_debug(self):
@@ -876,7 +905,7 @@ class SignaturesTests(unittest.TestCase):
                             **kwargs):
                     pass
 
-            parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
+            parser = ArgumentParser(exit_on_error=False, logger={'level': 'DEBUG'})
             with self.assertLogs(logger=parser.logger, level='DEBUG') as log:
                 parser.add_class_arguments(Class2, skip={'c2_a2'})
                 self.assertEqual(1, len(log.output))
@@ -894,7 +923,7 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, c1: Optional[Class1]):
                 self.c1 = c1
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_class_arguments(Class2)
 
         with mock_module(Class1) as module:
@@ -908,7 +937,7 @@ class SignaturesTests(unittest.TestCase):
             self.assertEqual(7, cfg['c1'].a1)
             self.assertEqual(2.3, cfg['c1'].a2)
 
-            parser = ArgumentParser(error_handler=None)
+            parser = ArgumentParser(exit_on_error=False)
             parser.add_class_arguments(Class2, 'c2')
 
             cfg = parser.parse_args(['--c2={"c1": {'+class_path+', '+init_args+'}}'])
@@ -919,7 +948,7 @@ class SignaturesTests(unittest.TestCase):
             class EmptyInitClass:
                 pass
 
-            parser = ArgumentParser(error_handler=None)
+            parser = ArgumentParser(exit_on_error=False)
             parser.add_class_arguments(EmptyInitClass, 'e')
             cfg = parser.parse_args([])
             cfg = parser.instantiate_classes(cfg)
@@ -949,18 +978,37 @@ class SignaturesTests(unittest.TestCase):
         def func(a1: Optional[int] = None):
             return a1
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_function_arguments(func)
 
         self.assertIsNone(parser.parse_args(['--a1=null']).a1)
 
 
+    def test_fail_untyped_true(self):
+        def func1(a1):
+            return a1
+
+        parser = ArgumentParser(exit_on_error=False)
+        with self.assertRaises(ValueError) as ctx:
+            parser.add_function_arguments(func1, fail_untyped=True)
+        self.assertIn('Parameter "a1" from', str(ctx.exception))
+        self.assertIn('does not specify a type', str(ctx.exception))
+
+        def func2(a2: 'int'):
+            return a2
+
+        with self.assertRaises(ValueError) as ctx:
+            parser.add_function_arguments(func2, fail_untyped=True)
+        self.assertIn('Parameter "a2" from', str(ctx.exception))
+        self.assertIn('specifies the type as a string', str(ctx.exception))
+
+
     def test_fail_untyped_false(self):
 
         def func(a1, a2=None):
             return a1
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         added_args = parser.add_function_arguments(func, fail_untyped=False)
 
         self.assertEqual(['a1', 'a2'], added_args)
@@ -976,7 +1024,7 @@ class SignaturesTests(unittest.TestCase):
             return c1
 
         with mock_module(Class1) as module:
-            parser = ArgumentParser(error_handler=None)
+            parser = ArgumentParser(exit_on_error=False)
             parser.add_function_arguments(func, fail_untyped=False)
 
             help_str = StringIO()
@@ -998,7 +1046,7 @@ class SignaturesTests(unittest.TestCase):
         with unittest.mock.patch('docstring_parser.parse') as docstring_parse:
             from docstring_parser import ParseError
             docstring_parse.side_effect = ParseError
-            parser = ArgumentParser(error_handler=None)
+            parser = ArgumentParser(exit_on_error=False)
             parser.add_class_arguments(Class1)
 
             help_str = StringIO()
@@ -1047,7 +1095,7 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, arg1: int, arg2: int = 1):
                 pass
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_argument("--config", action=ActionConfigFile)
         parser.add_class_arguments(Class, 'class')
         parser.add_subclass_arguments(BaseClass, 'subclass')
@@ -1106,7 +1154,7 @@ class SignaturesTests(unittest.TestCase):
 
         get_calendar_class = class_from_function(get_calendar)
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_class_arguments(get_calendar_class, 'a')
 
         if docstring_parser_support:
@@ -1152,7 +1200,7 @@ class SignaturesTests(unittest.TestCase):
                         class_path: {module}.Module
             """
 
-            parser = ArgumentParser(error_handler=None)
+            parser = ArgumentParser(exit_on_error=False)
             parser.add_argument('--config', action=ActionConfigFile)
             parser.add_class_arguments(Model, 'model')
 
@@ -1174,7 +1222,7 @@ class SignaturesTests(unittest.TestCase):
             with redirect_stderr(err), self.assertRaises(SystemExit):
                 parser.parse_args(['--cls=Class', '--cls.init_args.val=abc'])
             expected = textwrap.dedent('''
-            - Parser key "val":
+            Parser key "val":
               Does not validate against any of the Union subtypes
               Subtypes: (<class 'int'>, <class 'dict'>, <class 'NoneType'>)
               Errors:
@@ -1188,6 +1236,34 @@ class SignaturesTests(unittest.TestCase):
             self.assertIn(expected, err.getvalue())
 
 
+    def test_subclass_in_union_error_message_indentation(self):
+        class Class:
+            def __init__(self, val: int):
+                pass
+
+        with mock_module(Class):
+            parser = ArgumentParser()
+            parser.add_argument('--union', type=Union[str, Class])
+            err = StringIO()
+            with redirect_stderr(err), self.assertRaises(SystemExit):
+                parser.parse_object({'union': [{'class_path': 'Class', 'init_args': {'val': 'x'}}]})
+            expected = textwrap.dedent('''
+            Errors:
+              - Expected a <class 'str'>
+              - Not a valid subclass of Class
+                Subclass types expect one of:
+                - a class path (str)
+                - a dict with class_path entry
+                - a dict without class_path but with init_args entry (class path given previously)
+            Given value type: <class 'list'>
+            Given value: [{'class_path': 'Class', 'init_args': {'val': 'x'}}]
+            ''').strip()
+            expected = textwrap.indent(expected, '  ')
+            expected = '\n'.join(expected.splitlines())
+            obtained = '\n'.join(err.getvalue().splitlines())
+            self.assertIn(expected, obtained)
+
+
 class SignaturesConfigTests(TempDirTestCase):
 
     def test_add_function_arguments_config(self):
@@ -1197,7 +1273,7 @@ class SignaturesConfigTests(TempDirTestCase):
                  a3: bool = False):
             return a1
 
-        parser = ArgumentParser(error_handler=None, default_meta=False)
+        parser = ArgumentParser(exit_on_error=False, default_meta=False)
         parser.add_function_arguments(func, 'func')
 
         cfg_path = 'config.yaml'
@@ -1210,7 +1286,7 @@ class SignaturesConfigTests(TempDirTestCase):
         cfg = parser.parse_args(['--func={"a1": "ONE"}'])
         self.assertEqual(cfg.func, Namespace(a1='ONE', a2=2.0, a3=False))
 
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--func="""']))
+        self.assertRaises(ArgumentError, lambda: parser.parse_args(['--func="""']))
 
 
     def test_config_within_config(self):
@@ -1220,7 +1296,7 @@ class SignaturesConfigTests(TempDirTestCase):
                  a3: bool = False):
             return a1
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_argument('--cfg', action=ActionConfigFile)
         parser.add_function_arguments(func, 'func')
 
@@ -1239,7 +1315,7 @@ class SignaturesConfigTests(TempDirTestCase):
 
 
     def test_add_subclass_arguments_with_config(self):
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_argument('--cfg', action=ActionConfigFile)
         parser.add_subclass_arguments(Calendar, 'cal')
 
@@ -1255,7 +1331,7 @@ class SignaturesConfigTests(TempDirTestCase):
         cfg = parser.parse_args(['--cfg='+cfg_path, '--cal.init_args.firstweekday=2'])
         self.assertEqual(cfg['cal'].as_dict(), cal)
 
-        parser = ArgumentParser(error_handler=None, default_config_files=['config.yaml'])
+        parser = ArgumentParser(exit_on_error=False, default_config_files=['config.yaml'])
         parser.add_subclass_arguments(Calendar, 'cal')
 
         cfg = parser.parse_args(['--cal.init_args.firstweekday=2'])
@@ -1267,18 +1343,18 @@ class SignaturesConfigTests(TempDirTestCase):
             def __init__(self, param: int):
                 self.param = param
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_class_arguments(A, 'a')
         try:
             parser.parse_args(['--a=does_not_exist.yaml'])
-        except ParserError as ex:
+        except ArgumentError as ex:
             self.assertIn('Unable to load config "does_not_exist.yaml"', str(ex))
         else:
-            raise ValueError('Expected ParserError to be raised')
+            raise ValueError('Expected ArgumentError to be raised')
 
 
     def test_add_subclass_arguments_with_multifile_save(self):
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(exit_on_error=False)
         parser.add_subclass_arguments(Calendar, 'cal')
 
         cal_cfg_path = 'cal.yaml'
@@ -1322,7 +1398,7 @@ class SignaturesConfigTests(TempDirTestCase):
             with open('defaults.yaml', 'w') as f:
                 f.write(defaults)
 
-            parser = ArgumentParser(error_handler=None, default_config_files=['defaults.yaml'])
+            parser = ArgumentParser(exit_on_error=False, default_config_files=['defaults.yaml'])
             parser.add_class_arguments(Model, 'model')
 
             cfg = parser.parse_args(['--model.sub_module.init_args.p2=7'])
@@ -1339,7 +1415,7 @@ class SignaturesConfigTests(TempDirTestCase):
             def __init__(self, foo: int):
                 self.foo = foo
 
-        parser = ArgumentParser(default_config_files=["default.yaml"], error_handler=None)
+        parser = ArgumentParser(default_config_files=["default.yaml"], exit_on_error=False)
         parser.add_argument('--config', action=ActionConfigFile)
         subcommands = parser.add_subcommands()
 
@@ -1382,7 +1458,7 @@ class SignaturesConfigTests(TempDirTestCase):
 
             for subtest in ['class', 'subclass']:
                 with self.subTest(subtest):
-                    parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
+                    parser = ArgumentParser(exit_on_error=False, logger={'level': 'DEBUG'})
                     parser.add_argument('--config', action=ActionConfigFile)
 
                     if subtest == 'class':
@@ -1408,226 +1484,46 @@ class SignaturesConfigTests(TempDirTestCase):
                     self.assertIsInstance(init.main.sub, Sub2)
                     self.assertTrue(any("discarding init_args: {'s1': 'x'}" in o for o in log.output))
 
+    def test_config_nested_dict_discard_init_args(self):
+        class Base:
+            def __init__(self, b: float = 0.5):
+                pass
 
-@dataclasses.dataclass(frozen=True)
-class MyDataClassA:
-    """MyDataClassA description
+        class Sub1(Base):
+            def __init__(self, s1: int = 3, **kwargs):
+                super().__init__(**kwargs)
 
-    Args:
-        a1: a1 help
-        a2: a2 help
-    """
-    a1: PositiveInt = PositiveInt(1)  # type: ignore
-    a2: str = '2'
+        class Sub2(Base):
+            def __init__(self, s2: int = 4, **kwargs):
+                super().__init__(**kwargs)
 
-@dataclasses.dataclass
-class MyDataClassB:
-    """MyDataClassB description
+        class Main:
+            def __init__(self, sub: Optional[Dict] = None) -> None:
+                self.sub = sub
 
-    Args:
-        b1: b1 help
-        b2: b2 help
-    """
-    b1: PositiveFloat = PositiveFloat(3.0)  # type: ignore
-    b2: MyDataClassA = MyDataClassA()
+        configs, subconfigs, config_paths = {}, {}, {}
+        with mock_module(Base, Sub1, Sub2, Main) as module:
+            parser = ArgumentParser(exit_on_error=False, logger={'level': 'DEBUG'})
+            parser.add_argument('--config', action=ActionConfigFile)
+            parser.add_subclass_arguments(Main, 'main')
+            parser.set_defaults(main=lazy_instance(Main))
+            for c in [1, 2]:
+                subconfigs[c] = {
+                    'sub': {
+                        'class_path': f'{module}.Sub{c}',
+                        'init_args': {f's{c}': c},
+                    }
+                }
+                configs[c] = {'main': {'class_path': f'{module}.Main','init_args': subconfigs[c],}}
+                config_paths[c] = Path(f'config{c}.yaml')
+                config_paths[c].write_text(yaml.safe_dump(configs[c]))
 
-
-class DataclassesTests(unittest.TestCase):
-
-    @unittest.skipIf(not docstring_parser_support, 'docstring-parser package is required')
-    def test_add_dataclass_arguments(self):
-        parser = ArgumentParser(error_handler=None)
-        parser.add_dataclass_arguments(MyDataClassA, 'a', default=MyDataClassA(), title='CustomA title')
-        parser.add_dataclass_arguments(MyDataClassB, 'b', default=MyDataClassB())
-
-        cfg = parser.get_defaults()
-        self.assertEqual(dataclasses.asdict(MyDataClassA()), cfg['a'].as_dict())
-        self.assertEqual(dataclasses.asdict(MyDataClassB()), cfg['b'].as_dict())
-        dump = yaml.safe_load(parser.dump(cfg))
-        self.assertEqual(dataclasses.asdict(MyDataClassA()), dump['a'])
-        self.assertEqual(dataclasses.asdict(MyDataClassB()), dump['b'])
-
-        cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['a'], MyDataClassA)
-        self.assertIsInstance(cfg['b'], MyDataClassB)
-        self.assertIsInstance(cfg['b'].b2, MyDataClassA)
-
-        self.assertEqual(5, parser.parse_args(['--b.b2.a1=5']).b.b2.a1)
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--b.b2.a1=x']))
-
-        help_str = StringIO()
-        parser.print_help(help_str)
-        self.assertIn('CustomA title:', help_str.getvalue())
-        self.assertIn('MyDataClassB description:', help_str.getvalue())
-        self.assertIn('b2 help:', help_str.getvalue())
-
-        with self.assertRaises(ValueError):
-            parser.add_dataclass_arguments(1, 'c')
-
-        with self.assertRaises(ValueError):
-            parser.add_dataclass_arguments(MyDataClassB, 'c', default=MyDataClassB(b2=MyDataClassB()))
-
-        class MyClass(int, MyDataClassA):
-            """MyClass description"""
-
-        with self.assertRaises(ValueError):
-            parser.add_dataclass_arguments(MyClass, 'c')
-
-
-    def test_dataclass_typehint(self):
-
-        class MyClass:
-            def __init__(
-                self,
-                a1: MyDataClassA = MyDataClassA(),
-                a2: MyDataClassB = MyDataClassB(),
-            ):
-                self.a1 = a1
-                self.a2 = a2
-
-        parser = ArgumentParser()
-        parser.add_class_arguments(MyClass, 'g')
-
-        cfg = parser.get_defaults()
-        self.assertEqual(dataclasses.asdict(MyDataClassA()), cfg['g']['a1'].as_dict())
-        self.assertEqual(dataclasses.asdict(MyDataClassB()), cfg['g']['a2'].as_dict())
-        dump = yaml.safe_load(parser.dump(cfg))
-        self.assertEqual(dataclasses.asdict(MyDataClassA()), dump['g']['a1'])
-        self.assertEqual(dataclasses.asdict(MyDataClassB()), dump['g']['a2'])
-
-        cfg_init = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg_init.g.a1, MyDataClassA)
-        self.assertIsInstance(cfg_init.g.a2, MyDataClassB)
-        self.assertIsInstance(cfg_init.g.a2.b2, MyDataClassA)
-
-
-    def test_dataclass_typehint_in_subclass(self):
-
-        class MyClass1:
-            def __init__(self, a1: MyDataClassB = MyDataClassB()):
-                """MyClass1 description"""
-                self.a1 = a1
-
-        class MyClass2:
-            def __init__(self, c1: MyClass1):
-                """MyClass2 description"""
-                self.c1 = c1
-
-        parser = ArgumentParser(error_handler=None)
-        parser.add_class_arguments(MyClass2)
-
-        with mock_module(MyClass1, MyClass2) as module:
-            class_path = f'"class_path": "{module}.MyClass1"'
-            init_args = '"init_args": {"a1": {"b2": {"a1": 7}}}'
-            cfg = parser.parse_args(['--c1={'+class_path+', '+init_args+'}'])
-            self.assertEqual(cfg.c1.class_path, f'{module}.MyClass1')
-            self.assertEqual(cfg.c1.init_args.a1.b2.a1, 7)
-            self.assertIsInstance(cfg.c1.init_args.a1.b2.a1, PositiveInt)
-            cfg = parser.instantiate_classes(cfg)
-            self.assertIsInstance(cfg['c1'], MyClass1)
-            self.assertIsInstance(cfg['c1'].a1, MyDataClassB)
-            self.assertIsInstance(cfg['c1'].a1.b2, MyDataClassA)
-            self.assertIsInstance(cfg['c1'].a1.b1, PositiveFloat)
-
-
-    def test_dataclass_add_argument_type(self):
-        parser = ArgumentParser()
-        parser.add_argument('--b', type=MyDataClassB, default=MyDataClassB(b1=7.0))
-
-        cfg = parser.get_defaults()
-        self.assertEqual({'b1': 7.0, 'b2': {'a1': 1, 'a2': '2'}}, cfg['b'].as_dict())
-
-        cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['b'], MyDataClassB)
-        self.assertIsInstance(cfg['b'].b2, MyDataClassA)
-
-
-    def test_dataclass_add_argument_type_some_required(self):
-
-        @dataclasses.dataclass
-        class MyDataClass:
-            a1: str
-            a2: float = 1.2
-
-        parser = ArgumentParser(error_handler=None)
-        parser.add_argument('--b', type=MyDataClass)
-
-        self.assertEqual(Namespace(a1='v', a2=1.2), parser.parse_args(['--b.a1=v']).b)
-        self.assertRaises(ParserError, lambda: parser.parse_args([]))
-
-
-    def test_dataclass_field_default_factory(self):
-
-        @dataclasses.dataclass
-        class MyDataClass:
-            a1: List[int] = dataclasses.field(default_factory=lambda: [1, 2, 3])
-            a2: Dict[str, float] = dataclasses.field(default_factory=lambda: {'a': 1.2, 'b': 3.4})
-
-        parser = ArgumentParser()
-        parser.add_class_arguments(MyDataClass)
-
-        cfg = parser.get_defaults().as_dict()
-        self.assertEqual([1, 2, 3], cfg['a1'])
-        self.assertEqual({'a': 1.2, 'b': 3.4}, cfg['a2'])
-
-
-    def test_compose_dataclasses(self):
-
-        @dataclasses.dataclass
-        class MyDataClassA:
-            a: int = 1
-
-            def __post_init__(self):
-                self.a += 1
-
-        @dataclasses.dataclass
-        class MyDataClassB:
-            b: str = '1'
-
-        MyDataClassAB = compose_dataclasses(MyDataClassA, MyDataClassB)
-        self.assertEqual(2, len(dataclasses.fields(MyDataClassAB)))
-        self.assertEqual({'a': 3, 'b': '2'}, dataclasses.asdict(MyDataClassAB(a=2, b='2')))  # pylint: disable=unexpected-keyword-arg
-
-
-    def test_instantiate_classes_dataclasses_lightning_issue_9207(self):
-        # https://github.com/PyTorchLightning/pytorch-lightning/issues/9207
-
-        @dataclasses.dataclass
-        class MyDataClass:
-            name: str = 'name'
-
-        class MyClass:
-            def __init__(self, data: MyDataClass):
-                self.data = data
-
-        parser = ArgumentParser()
-        parser.add_class_arguments(MyClass, 'class')
-        cfg = parser.parse_args([])
-        cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['class'], MyClass)
-        self.assertIsInstance(cfg['class'].data, MyDataClass)
-
-
-    @unittest.skipIf(not docstring_parser_support, 'docstring-parser package is required')
-    def test_attribute_docstrings(self):
-
-        @dataclasses.dataclass
-        class WithAttrDocs:
-            attr_str: str = 'a'
-            "attr_str description"
-            attr_int: int = 1
-            "attr_int description"
-
-        set_docstring_parse_options(attribute_docstrings=True)
-
-        parser = ArgumentParser()
-        parser.add_class_arguments(WithAttrDocs)
-        help_str = StringIO()
-        parser.print_help(help_str)
-        self.assertIn('attr_str description (type: str, default: a)', help_str.getvalue())
-        self.assertIn('attr_int description (type: int, default: 1)', help_str.getvalue())
-
-        set_docstring_parse_options(attribute_docstrings=False)
+            with self.assertLogs(logger=parser.logger, level='DEBUG') as log:
+                cfg = parser.parse_args([f'--config={config_paths[1]}', f'--config={config_paths[2]}'])
+            init = parser.instantiate_classes(cfg)
+            self.assertIsInstance(init.main, Main)
+            self.assertTrue(init.main.sub['init_args']['s2'], 2)
+            self.assertTrue(any("discarding init_args: {'s1': 1}" in o for o in log.output))
 
 
 if __name__ == '__main__':
