@@ -68,8 +68,8 @@ class PathTests(TempDirTestCase):
         path1 = Path(self.file_rw, 'frw')
         path2 = Path(path1)
         self.assertEqual(path1.cwd, path2.cwd)
-        self.assertEqual(path1.abs_path, path2.abs_path)
-        self.assertEqual(path1.rel_path, path2.rel_path)
+        self.assertEqual(path1.absolute, path2.absolute)
+        self.assertEqual(path1.relative, path2.relative)
         self.assertEqual(path1.is_url, path2.is_url)
         self.assertRaises(TypeError, lambda: Path(True))
         self.assertRaises(ValueError, lambda: Path(self.file_rw, '-'))
